library getxfire;

import 'dart:convert';
import 'dart:io';
import 'dart:ui' as ui;
import 'dart:async';
import 'package:crypto/crypto.dart';
import 'package:dio/dio.dart';
import 'package:firebase_core/firebase_core.dart';
// import 'package:firebase_messaging/firebase_messaging.dart';
import 'package:firebase_storage/firebase_storage.dart';
import 'package:flutter/foundation.dart';
import 'package:flutter/material.dart';
import 'package:firebase_auth/firebase_auth.dart';
import 'package:cloud_firestore/cloud_firestore.dart';
// import 'package:flutter_facebook_auth/flutter_facebook_auth.dart';
import 'package:flutter_image_compress/flutter_image_compress.dart';
import 'package:google_sign_in/google_sign_in.dart';
import 'package:image_picker/image_picker.dart';
import 'package:merge_map/merge_map.dart';
import 'package:permission_handler/permission_handler.dart' as permissionHander;
import 'package:rxdart/subjects.dart';
import 'package:algolia/algolia.dart';

import 'package:location/location.dart';
import 'package:geoflutterfire/geoflutterfire.dart';

// import 'src/functions.dart';
part 'src/auth.dart';
// part 'src/chats.dart';
// part 'src/definitions.dart';
// part 'src/locations.dart';

final FirebaseAuth _auth = FirebaseAuth.instance;

/// GetxFire
///
/// Recommendation: instantiate `GetxFire` class into a global variable
/// and use it all over the app runtime.
///
/// Warning: instantiate it after `initFirebase`. One of good places is insdie
/// the first widget loaded by `runApp()` or home screen.
///
///
class GetxFire {
  ///
  /// Returns an instance using the default [FirebaseApp].
  static FirebaseAuth get auth => _auth;

  ///
  /// Returns the current [User] if they are currently signed-in, or `null` if
  /// not.
  ///
  /// You should not use this getter to determine the users current state,
  /// instead use [authStateChanges], [idTokenChanges] or [userChanges] to
  /// subscribe to updates.
  ///
  static User? get currentUser => _auth.currentUser;

  ///
  ///Initializes a new [FirebaseApp] instance by [name] and [options] and returns the created app. This method should be called before any usage of FlutterFire plugins.
  /// The default app instance cannot be initialized here and should be created using the platform Firebase integration.
  ///
  /// [remoteConfigFetchInterval] is the interval that the app will fetch remote config data again.
  /// the unit of [remoteConfigFetchInterval] is minute. In debug mode, it can be set to 1.
  /// But in release mode, it must not be less than 15. If it is less than 15,
  /// then it will be escalated to 15.
  ///
  static Future<FirebaseApp> init({
    String? name,
    FirebaseOptions? options,
    // bool openProfile = false,
    // bool enableNotification = false,
    // String? firebaseServerToken,
    // Map<String, Map<dynamic, dynamic>>? settings,
    // Map<String, Map<String, String>>? translations,
  }) async {
    // this.openProfile = openProfile;
    // this.enableNotification = enableNotification;
    // this.firebaseServerToken = firebaseServerToken;

    /// Initialize settings.
    ///
    /// Note. it must be called before firebase init.
    // if (settings != null) {
    //   _settings = mergeMap([_settings!, settings]);
    //   settingsChange.add(_settings);
    // }

    // if (translations != null) {
    //   translationsChange
    //       .add(translations); // Must be called before firebase init
    // }

    ///
    return Firebase.initializeApp(name: name, options: options)
        .then((firebaseApp) {
      FirebaseFirestore.instance.settings =
          Settings(cacheSizeBytes: Settings.CACHE_SIZE_UNLIMITED);

      // usersCol = FirebaseFirestore.instance.collection('users');
      // postsCol = FirebaseFirestore.instance.collection('posts');

      // initUser();
      // // initFirebaseMessaging();
      // listenSettingsChange();
      // listenTranslationsChange(translations);

      // /// Initialize or Re-initialize based on the setting's update.
      // settingsChange.listen((settings) {
      //   // print('settingsChange.listen() on GetxFire::init() $settings');

      //   // Initalize Algolia
      //   String? algoliaAppId = appSetting(ALGOLIA_APP_ID);
      //   String? apiKey = appSetting(ALGOLIA_ADMIN_API_KEY);
      //   if (algoliaAppId != null && apiKey != null) {
      //     algolia = Algolia.init(
      //       applicationId: algoliaAppId,
      //       apiKey: apiKey,
      //     );
      //   }
      // });

      // isFirebaseInitialized = true;
      // firebaseInitialized.add(isFirebaseInitialized);
      return firebaseApp;
    });
  }

  /// Logs in signInAnonymously
  ///
  static Future<UserCredential?> signInAnonymously({
    Function(UserCredential? userCredential)? onSuccess,
    Function(String? code, String? message)? onError,
  }) async =>
      Auth.signInAnonymously(onSuccess: onSuccess, onError: onError);

  /// Logs into Firebase Auth.
  ///
  /// It can update user displayName, photoURL or other public data while login.
  ///
  /// TODO Leave last login timestamp.
  /// TODO Increment login count
  /// TODO Leave last login device & IP address.
  ///
  static Future<UserCredential?> signInWithEmailAndPassword({
    required String email,
    required String password,
    Map<String, dynamic>? data,
    Map<String, dynamic>? public,
    Function(UserCredential? userCredential)? onSuccess,
    Function(String? code, String? message)? onError,
  }) async =>
      Auth.signInWithEmailAndPassword(
        email: email,
        password: password,
        data: data,
        public: public,
        onSuccess: onSuccess,
        onError: onError,
      );

  /// create User With Email And Password in Firebase Auth.
  ///
  /// It can update user displayName, photoURL or other public data while register.
  ///
  static Future<UserCredential?> createUserWithEmailAndPassword({
    required String email,
    required String password,
    Map<String, dynamic>? data,
    Map<String, dynamic>? public,
    Function(UserCredential? userCredential)? onSuccess,
    Function(String? code, String? message)? onError,
  }) async =>
      Auth.createUserWithEmailAndPassword(
        email: email,
        password: password,
        data: data,
        public: public,
        onSuccess: onSuccess,
        onError: onError,
      );

  /// Logs out from Firebase Auth and All Social Login.
  static Future<void> signOut({bool isSocialLogout = true}) async {
    if (isSocialLogout) await GoogleSignIn().signOut();
    return await FirebaseAuth.instance.signOut();
  }
}
